@@ -20,5 +20,5 @@ from .documententry import DocumentEntry
 from .documentstore import DocumentStore
 from .documentrepository import DocumentRepository
 from .pdfdocumentrepository import PDFDocumentRepository
-from .compositerepository import CompositeRepository
-__version__ = "0.1.6.1" #gets pulled into setup.py and docs/conf.py
+from .compositerepository import CompositeRepository, CompositeStore
+__version__ = "0.1.7" #gets pulled into setup.py and docs/conf.py
