@@ -4,10 +4,10 @@ from __future__ import unicode_literals
 from contextlib import contextmanager
 import shutil
 import os
-import sys
 from tempfile import NamedTemporaryFile
 import filecmp
 
+import six
 from six.moves.urllib_parse import quote, unquote
 
 from ferenda import util
@@ -68,7 +68,7 @@ class DocumentStore(object):
             fp = open(filename, mode)
             yield fp
 
-    def path(self, basefile, maindir, suffix, version=None, attachment=None):
+    def path(self, basefile, maindir, suffix, version=None, attachment=None, storage_policy=None):
         """Calculate a full filesystem path for the given parameters.
 
         :param basefile: The basefile of the resource we're calculating a filename for
@@ -80,6 +80,8 @@ class DocumentStore(object):
         :type  version: str
         :param attachment: Optional. Any associated file needed by the main file. 
         :type  attachment: str
+        :param storage_policy: Optional. Used to override `storage_policy` if needed
+        :type  attachment: str
 
         .. note::
 
@@ -119,6 +121,9 @@ class DocumentStore(object):
         """
         pathfrag = self.basefile_to_pathfrag(basefile)
 
+        if not storage_policy:
+            storage_policy = self.storage_policy
+        
         if version:
             v_pathfrag = self.basefile_to_pathfrag(version)
             segments = [self.datadir,
@@ -126,7 +131,7 @@ class DocumentStore(object):
         else:
             segments = [self.datadir, maindir, pathfrag]
 
-        if self.storage_policy == "dir":
+        if storage_policy == "dir":
             if attachment:
                 for illegal in ':/':
                     if illegal in attachment:
@@ -139,7 +144,7 @@ class DocumentStore(object):
             if attachment != None:
                 raise errors.AttachmentPolicyError(
                     "Can't add attachments (name %s) if "
-                    "self.storage_policy != 'dir'" % attachment)
+                    "storage_policy != 'dir'" % attachment)
             segments[-1] += suffix
 
         unixpath = "/".join(segments)
@@ -208,7 +213,7 @@ class DocumentStore(object):
                 # convention, the main file is called index.html,
                 # index.pdf or whatever.
                 # print("storage_policy dir: %s" % self.storage_policy)
-                suffix = "index" + self.downloaded_suffix
+                suffix = os.sep + "index" + self.downloaded_suffix
             else:
                 # print("storage_policy file: %s" % self.storage_policy)
                 suffix = self.downloaded_suffix
@@ -218,7 +223,7 @@ class DocumentStore(object):
         elif action == "generate":
             directory = os.path.sep.join((basedir, "parsed"))
             if self.storage_policy == "dir":
-                suffix = "index.xhtml"
+                suffix = os.sep + "index.xhtml"
             else:
                 suffix = ".xhtml"
         elif action == "news":
@@ -241,10 +246,10 @@ class DocumentStore(object):
             # ignore empty files placed by download (which may
             # have done that in order to avoid trying to
             # re-download nonexistent resources)
-
             if os.path.exists(x) and os.path.getsize(x) > 0:
                 # get a pathfrag from full path
-                suffixlen = len(suffix) if self.storage_policy == "file" else len(suffix) + 1
+                # suffixlen = len(suffix) if self.storage_policy == "file" else len(suffix) + 1
+                suffixlen = len(suffix) 
                 x = x[len(directory) + 1:-suffixlen]
                 yield self.pathfrag_to_basefile(x)
 
@@ -313,11 +318,14 @@ class DocumentStore(object):
         :returns: All available attachments for the basefile 
         :rtype: generator
         """
-
+        if self.storage_policy != "dir":
+            raise errors.AttachmentPolicyError("Can't list attachments if storage_policy != 'dir'")
+        
         basedir = self.datadir
-        pathfrag = self.pathfrag_to_basefile(basefile)
+        # pathfrag = self.pathfrag_to_basefile(basefile) # that can't be right?
+        pathfrag = self.basefile_to_pathfrag(basefile)
         if version:
-            v_pathfrag = self.pathfrag_to_basefile(version)
+            v_pathfrag = self.basefile_to_pathfrag(version)
             directory = os.sep.join((basedir, "archive", action, pathfrag, v_pathfrag))
         else:
             directory = os.sep.join((basedir, action, pathfrag))
@@ -363,12 +371,15 @@ class DocumentStore(object):
         :rtype: str
         """
         safe = '/;@&=+,'
-        if sys.version_info < (2, 7, 0):
-            # urllib.quote in python 2.6 cannot handle unicode values
-            # for the safe parameter. FIXME: We should create a shim
-            # as ferenda.compat.quote and use that
-            safe = safe.encode('ascii') # pragma: no cover
 
+        if six.PY2:
+            # urllib.quote in python 2 cannot handle unicode values
+            # for the s parameter (2.6 cannot even handle unicode
+            # values for the safe parameter). FIXME: We should create
+            # a shim as ferenda.compat.quote and use that
+            basefile = basefile.encode('utf-8')
+            safe = safe.encode('ascii') # pragma: no cover
+            
         return quote(basefile, safe=safe).replace('%', os.sep + '%')
 
     def pathfrag_to_basefile(self, pathfrag):
@@ -398,13 +409,16 @@ class DocumentStore(object):
         """
 
         for meth in (self.downloaded_path, self.documententry_path,
-                     self.parsed_path, self.distilled_path,
+                     self.parsed_path, self.serialized_path,
+                     self.distilled_path,
                      self.annotation_path, self.generated_path):
             # FIXME: what about intermediate? Ignore them as they
             # should be able to be regenerated at any time?
             src = meth(basefile)
             dest = meth(basefile, version)
-            if self.storage_policy == "dir":
+            if self.storage_policy == "dir" and meth in (self.downloaded_path,
+                                                         self.parsed_path,
+                                                         self.generated_path):
                 src = os.path.dirname(src)
                 dest = os.path.dirname(dest)
             if not os.path.exists(src):
@@ -446,7 +460,7 @@ class DocumentStore(object):
         return self._open(filename, mode)
 
     def documententry_path(self, basefile, version=None):
-        """Get the full path for the documententry file for the given
+        """Get the full path for the documententry JSON file for the given
         basefile (and optionally archived version).
 
         :param basefile: The basefile for which to calculate the path
@@ -456,7 +470,7 @@ class DocumentStore(object):
         :returns: The full filesystem path
         :rtype:   str
         """
-        return self.path(basefile, 'entries', '.json', version)
+        return self.path(basefile, 'entries', '.json', version, storage_policy="file")
 
     def intermediate_path(self, basefile, version=None, attachment=None):
         """Get the full path for the main intermediate file for the given
@@ -473,8 +487,17 @@ class DocumentStore(object):
         """
         return self.path(basefile, 'intermediate', '.xml', version, attachment)
 
+    def open_intermediate(self, basefile, mode="r", version=None, attachment=None):
+        """Opens files for reading and writing,
+        c.f. :meth:`~ferenda.DocumentStore.open`. The parameters are
+        the same as for
+        :meth:`~ferenda.DocumentStore.intermediate_path`.
+        """
+        filename = self.intermediate_path(basefile, version, attachment)
+        return self._open(filename, mode)
+
     def parsed_path(self, basefile, version=None, attachment=None):
-        """Get the full path for the parsed file for the given
+        """Get the full path for the parsed XHTML file for the given
         basefile.
 
         :param basefile: The basefile for which to calculate the path
@@ -501,6 +524,30 @@ class DocumentStore(object):
         filename = self.parsed_path(basefile, version, attachment)
         return self._open(filename, mode)
 
+    def serialized_path(self, basefile, version=None, attachment=None):
+        """Get the full path for the serialized JSON file for the given
+        basefile.
+
+        :param basefile: The basefile for which to calculate the path
+        :type  basefile: str
+        :param  version: Optional. The archived version id
+        :type   version: str
+        :returns: The full filesystem path
+        :rtype:   str
+        """
+        return self.path(basefile, 'serialized', '.json',
+                         version, storage_policy="file")
+
+    def open_serialized(self, basefile, mode="r", version=None):
+        """Opens files for reading and writing,
+        c.f. :meth:`~ferenda.DocumentStore.open`. The parameters are
+        the same as for
+        :meth:`~ferenda.DocumentStore.serialized_path`.
+
+        """
+        filename = self.serialized_path(basefile, version)
+        return self._open(filename, mode)
+
     def distilled_path(self, basefile, version=None):
         """Get the full path for the distilled RDF/XML file for the given
         basefile.
@@ -513,7 +560,7 @@ class DocumentStore(object):
         :rtype:   str
         """
         return self.path(basefile, 'distilled', '.rdf',
-                         version)
+                         version, storage_policy="file")
 
     def open_distilled(self, basefile, mode="r", version=None):
         """Opens files for reading and writing,
@@ -566,7 +613,7 @@ class DocumentStore(object):
         :rtype:   str
         """
         return self.path(basefile, 'annotations', '.grit.xml',
-                         version)
+                         version, storage_policy="file")
 
     def open_annotation(self, basefile, mode="r", version=None):
         """Opens files for reading and writing,
@@ -585,7 +632,7 @@ class DocumentStore(object):
         :returns: The full filesystem path
         :rtype:   str
         """
-        return self.path(basefile, 'deps', '.txt')
+        return self.path(basefile, 'deps', '.txt', storage_policy="file")
 
     def open_dependencies(self, basefile, mode="r"):
         """Opens files for reading and writing,
@@ -612,4 +659,4 @@ class DocumentStore(object):
         :rtype:   str
 
         """
-        return self.path(basefile, 'feed', '.atom')
+        return self.path(basefile, 'feed', '.atom', storage_policy="file")
