@@ -16,21 +16,21 @@ hierarchies to and from strings.
 
 """
 from __future__ import unicode_literals
-
-import datetime
-import re
-import sys
-import logging
+from operator import itemgetter
 import ast
+import datetime
+import json
+import logging
+import re
+import inspect
 import xml.etree.cElementTree as ET
+import sys
 
 from lxml.builder import ElementMaker
-from operator import itemgetter
-
-import six
-from six import text_type as str
-from six import binary_type as bytes
 from rdflib import Graph, Namespace, Literal, URIRef
+from six import binary_type as bytes
+from six import text_type as str
+import six
 import pyparsing
 
 from ferenda import util
@@ -42,17 +42,24 @@ log = logging.getLogger(__name__)
 E = ElementMaker(namespace="http://www.w3.org/1999/xhtml",
                  nsmap={None: "http://www.w3.org/1999/xhtml"})
 
-def serialize(root):
+def serialize(root, format="xml"):
     """Given any :py:class:`~ferenda.elements.AbstractElement` *root*
     object, returns a XML serialization of *root*, recursively.
 
     """
-    t = __serializeNode(root)
-    _indentTree(t)
-    return ET.tostring(t, 'utf-8').decode('utf-8') + "\n"
+    if format == "xml":
+        t = __serialize_xml(root)
+        _indentTree(t)
+        return ET.tostring(t, 'utf-8').decode('utf-8') + "\n"
+    elif format == "json":
+        t = __serialize_json(root)
+        r = json.dumps(t, indent=4, ensure_ascii=False, sort_keys=True)
+        return r
+    else:
+        raise ValueError("Invalid serialization format: %s" % format)
 
 
-def deserialize(xmlstr, caller_globals):
+def deserialize(xmlstr, format="xml", caller_globals=None):
     """Given a XML string created by :py:func:`serialize`, returns a
     object tree of :py:class:`AbstractElement` derived objects that is
     identical to the initial object structure.
@@ -67,10 +74,16 @@ def deserialize(xmlstr, caller_globals):
     # print "Callee globals()"
     # print repr(globals().keys())
     # print repr(locals().keys())
-    if (isinstance(xmlstr, str)):
-        xmlstr = xmlstr.encode('utf-8')
-    t = ET.fromstring(xmlstr)
-    return  __deserializeNode(t, caller_globals)
+    if format == "xml":
+        if (isinstance(xmlstr, str)):
+            xmlstr = xmlstr.encode('utf-8')
+        t = ET.fromstring(xmlstr)
+        return  __deserialize_xml(t, caller_globals)
+    elif format=="json":
+        root = json.loads(xmlstr)
+        t = __deserialize_json(root)
+        return t
+        
 
 
 class AbstractElement(object):
@@ -225,7 +238,8 @@ class CompoundElement(AbstractElement, list):
             
         if hasattr(self,'uri') and self.uri and hasattr(self,'meta') and self.meta:
             assert isinstance(self.meta,Graph), "self.meta is %r, not rdflib.Graph" % type(self.meta)
-            # we sort to get a predictable order (by predicate)
+            # we use sorted() to get the triples in a predictable
+            # order (by predicate)
             for (s,p,o) in sorted(self.meta, key=itemgetter(1)):
                 if s != URIRef(self.uri):
                     continue
@@ -399,8 +413,14 @@ class Link(UnicodeElement):
     """A unicode string with also has a ``.uri`` attribute"""
     tagname = 'a'
 
+    # FIXME: can __repr__ on PY2 return unicode data?
     def __repr__(self):
-        return 'Link(\'%s\', uri=%s)' % (self, self.uri)
+        # convoluted way around a UnicodeEncode error on py2 when self contains non-ascii characters
+        if six.PY2:
+            rep = repr(str(self))[2:-1]
+        else:
+            rep = self
+        return 'Link(\'%s\', uri=%s)' % (rep, self.uri)
 
     def as_xhtml(self, uri):
         element = super(Link, self).as_xhtml(uri)
@@ -446,6 +466,13 @@ class SectionalElement(CompoundElement):
             newuri = self.uri
         else:
             newuri = baseuri + "#S%s" % self.ordinal
+        # Addressable subelements (ie. elements which has an uri) are
+        # expected to be able to construct that URI from the base uri
+        # of the entire document (plus it's own internal state). Even
+        # though we construct a new uri here for a particular
+        # subelement, we don't pass it along as the uri parameter for
+        # its own subelements -- these can construct their entire URI
+        # themselves.
         element = super(SectionalElement, self).as_xhtml(baseuri)
         if not hasattr(self, 'uri') or not hasattr(self, 'meta'):
             element.set('property', 'dct:title')
@@ -503,7 +530,146 @@ class ListItem(CompoundElement, OrdinalElement):
     tagname = 'li'
 
 
-def __serializeNode(node, serialize_hidden_attrs=False):
+def __serialize_json(node):
+    # some native datatypes should be returned as-is, ie not wrapped
+    # in a dict. Note that types derived from these gets handled
+    # differently
+    if type(node) in (str, int, bool):
+        return node
+    # NOTE: this transforms bytes into str's (even though on py2 these will be labeled as "str")
+    # elif type(node) == bytes: 
+    #     return node.decode()
+    # lists and dicts gets returned as-is, but the values in those
+    # containers are transformed if need be
+    elif type(node) == list:
+        return [__serialize_json(x) for x in node]
+    elif type(node) == dict:
+        return dict([(k, __serialize_json(v)) for k,v in node.items()])
+    else:
+        if node.__class__.__module__ in ('builtins', '__builtin__'):
+            if type(node) == str:  # py2 workaround -- we want a str to be known as 'str' always, but py2 thinks they're called 'unicode'...
+                typename = "str"
+            elif type(node) == bytes:
+                typename = "bytes"
+            else:
+                typename = node.__class__.__name__
+        else:
+            typename = node.__class__.__module__ + "." + node.__class__.__name__
+
+        e = {'@class': typename}
+        if hasattr(node, '__dict__'):
+            for key in [x for x in list(node.__dict__.keys()) if not x.startswith('_')]:
+                val = node.__dict__[key]
+                if val is None:
+                    continue
+                elif isinstance(val, logging.Logger):
+                    continue
+                e[key] = __serialize_json(val)
+        if isinstance(node, list) or isinstance(node, tuple):
+            # convert derived list to plain list
+            e['@content'] = __serialize_json(list(node))
+        elif isinstance(node, dict):
+            # convert derived dict to plain dict
+            e['@content'] = __serialize_json(dict(node))
+        elif isinstance(node, bytes):
+            # assume that all bytestrings are ascii only. When this
+            # assumption does not hold, convert them into something
+            # like base64.
+            e['@content'] = node.decode()
+        elif isinstance(node, str):
+            e['@content'] = str(node)
+        elif isinstance(node, pyparsing.ParseResults):
+            e['name'] = node.getName()
+            e['@content'] = repr(node)
+        elif isinstance(node, Graph):
+            i = node.identifier
+            e['@content'] = node.serialize().decode('utf-8') # default compact RDF/XML
+            e['identifier'] = {'@class': i.__class__.__module__ + "." + i.__class__.__name__,
+                               '@content': str(i)}
+        else:
+            # if hasattr(node, '__json__'): ...
+            e['@content'] = repr(node)
+        return e
+
+_state = {'fontspec': None}
+def __deserialize_json(node):
+    from ferenda.pdfreader import PDFReader, Textbox
+    from ferenda import Document
+    nodetype = None
+    if hasattr(node, 'get'):
+        nodetype = node.get("@class")
+    # 1. get the appropriate class object
+    if nodetype:
+        try:
+            modulename, classname = nodetype.rsplit(".", 1)
+            __import__(modulename)
+            m = sys.modules[modulename]
+            for name, cls in inspect.getmembers(m, inspect.isclass):
+                if name == classname:
+                    break # setting cls to what we want
+        except ValueError: # "need more than 1 value to unpack": we have a builtin type like str, int, bool
+            cls = {'str':str,
+                   'int':int,
+                   'bool':bool,
+                   'bytes':bytes}[nodetype]
+        content = node['@content']
+    else:
+        # native objects (int, str, bool, list, dict (which is
+        # distinguishable from complex/derived objects by the absence
+        # of '@type'))
+        cls = node.__class__
+        content = node
+
+    # 2. get any possible attributes
+    attribs = {}
+    if hasattr(node, 'items'):
+        attribs = dict([(k,__deserialize_json(v)) for k, v in node.items() if not k.startswith("@")])
+
+    # hack for deserializing pdfreader objects -- store the current
+    # fontspecs dict as soon as we see it, we'll be needing it every
+    # time we create a pdfreader.Textbox
+
+
+    if issubclass(cls, PDFReader):
+        _state['fontspec'] =  attribs['fontspec']
+
+    # 3. initialize the class
+
+    
+    if issubclass(cls, list):
+        if issubclass(cls, Textbox):
+            attribs['fontspec'] = _state['fontspec']
+        o = cls([__deserialize_json(x) for x in content], **attribs)
+    elif issubclass(cls, dict):
+        o = cls([(k, __deserialize_json(v)) for k,v in content.items()], **attribs)
+    elif issubclass(cls, bytes):
+        # assume only ascii
+        o = cls(content.encode(), **attribs)
+    elif issubclass(cls, datetime.date):
+        m = re.match(r'[\w\.]+\((\d+), (\d+), (\d+)\)', content)
+        o = cls(int(m.group(1)), int(m.group(2)), int(m.group(3)), **attribs)
+    elif issubclass(cls, pyparsing.ParseResults):
+        # attempt to reconstruct a pyparsing.ParseResult object from
+        # its repr() serialization. It's not 100 %, but good enough to
+        # be usable.
+        tocdict = ast.literal_eval(content)
+        o = cls([])
+        parent = None
+        accumNames = {}
+        name = node['name']
+        o.__setstate__((tocdict[0], (tocdict[1], parent, accumNames, name)))
+    elif issubclass(cls, Graph):
+        # first create the graph identifier (BNode or URIRef object)
+        i = __deserialize_json(node['identifier'])
+        o = cls(identifier=i).parse(data=content)
+    elif issubclass(cls, Document):
+        o = cls(**attribs) # no actual content, only attributes/properties
+    else:
+        o = cls(content, **attribs)
+    return o
+
+
+def __serialize_xml(node, serialize_hidden_attrs=False):
     # print "serializing: %r" % node
 
     # Special handling of pyparsing.ParseResults -- deserializing of
@@ -541,13 +707,13 @@ def __serializeNode(node, serialize_hidden_attrs=False):
         e.text = str(node)
     elif isinstance(node, list):
         for x in node:
-            e.append(__serializeNode(x))
+            e.append(__serialize_xml(x))
     else:
         e.text = repr(node)
         # raise TypeError("Can't serialize %r (%r)" % (type(node), node))
     return e
 
-def __deserializeNode(elem, caller_globals):
+def __deserialize_xml(elem, caller_globals):
     # print "element %r, attrs %r" % (elem.tag, elem.attrib)
     # kwargs = elem.attrib
 
@@ -589,7 +755,7 @@ def __deserializeNode(elem, caller_globals):
         c = cls(**elem.attrib)
         for subelem in elem:
             # print "Recursing"
-            c.append(__deserializeNode(subelem, caller_globals))
+            c.append(__deserialize_xml(subelem, caller_globals))
 
     return c
 
