@@ -657,7 +657,18 @@ def setup_logger(level='INFO', filename=None,
     # turn of some library loggers we're not interested in
     for logname in ['requests.packages.urllib3.connectionpool',
                     'rdflib.plugins.sleepycat']:
-        logging.getLogger(logname).propagate = False
+        log = logging.getLogger(logname)
+        log.propagate = False
+        if log.handlers == []:
+            if hasattr(logging, 'NullHandler'):
+                log.addHandler(logging.NullHandler())
+            else:  # pragma: no cover
+                # py26 compatibility
+                class NullHandler(logging.Handler): 
+
+                    def emit(self, record):
+                        pass
+                log.addHandler(NullHandler())
 
     return l
 
@@ -690,7 +701,6 @@ def run(argv):
                  positional arguments to the specified action).
     """
     config = _load_config(_find_config_file(), argv)
-
     # if logfile is set to True, autogenerate logfile name from
     # current datetime. Otherwise assume logfile is set to the desired
     # file name of the log
@@ -704,78 +714,84 @@ def run(argv):
         util.ensure_dir(logfile)
         setup_logger(level=config.loglevel, filename=logfile)
 
-    # reads only ferenda.ini using configparser rather than layeredconfig
-    enabled = _enabled_classes()
-    # returns {'ferenda.sources.docrepo.DocRepo':'base',...}
-    enabled_aliases = dict(reversed(item) for item in enabled.items())
-    if len(argv) < 1:
-        _print_usage()  # also lists enabled modules
-    else:
-        # _filter_argv("ecj", "parse", "62008J0034", "--force=True", "--frobnicate")
-        #    -> ("ecj", "parse", ["62008J0034"])
-        # _filter_argv("ecj", "--frobnicate") -> ("ecj", None, [])
-        (classname, action, args) = _filter_argv(argv)
-        if action == 'enable':
-            try:
-                return enable(classname)
-            except (ImportError, ValueError) as e:
-                log.error(six.text_type(e))
-                return None
-        elif action == 'runserver':
-            args = _setup_runserver_args(config, _find_config_file())
-            # Note: the actual runserver method never returns
-            return runserver(**args)
+    try:
+        # reads only ferenda.ini using configparser rather than layeredconfig
+        enabled = _enabled_classes()
+        # returns {'ferenda.sources.docrepo.DocRepo':'base',...}
+        enabled_aliases = dict(reversed(item) for item in enabled.items())
+        if len(argv) < 1:
+            _print_usage()  # also lists enabled modules
+        else:
+            # _filter_argv("ecj", "parse", "62008J0034", "--force=True", "--frobnicate")
+            #    -> ("ecj", "parse", ["62008J0034"])
+            # _filter_argv("ecj", "--frobnicate") -> ("ecj", None, [])
+            (classname, action, args) = _filter_argv(argv)
+            if action == 'enable':
+                try:
+                    return enable(classname)
+                except (ImportError, ValueError) as e:
+                    log.error(six.text_type(e))
+                    return None
+            elif action == 'runserver':
+                args = _setup_runserver_args(config, _find_config_file())
+                # Note: the actual runserver method never returns
+                return runserver(**args)
 
-        elif action == 'makeresources':
-            repoclasses = _classes_from_classname(enabled, classname)
-            args = _setup_makeresources_args(config)
-            repos = []
-            for cls in repoclasses:
-                inst = _instantiate_class(cls, _find_config_file(), argv)
-                repos.append(inst)
-            return makeresources(repos, **args)
+            elif action == 'makeresources':
+                repoclasses = _classes_from_classname(enabled, classname)
+                args = _setup_makeresources_args(config)
+                repos = []
+                for cls in repoclasses:
+                    inst = _instantiate_class(cls, _find_config_file(), argv)
+                    repos.append(inst)
+                return makeresources(repos, **args)
 
-        elif action == 'frontpage':
-            repoclasses = _classes_from_classname(enabled, classname)
-            args = _setup_frontpage_args(config, argv)
-            return frontpage(**args)
+            elif action == 'frontpage':
+                repoclasses = _classes_from_classname(enabled, classname)
+                args = _setup_frontpage_args(config, argv)
+                return frontpage(**args)
 
-        elif action == 'all':
-            classnames = _setup_classnames(enabled, classname)
-            results = OrderedDict()
-            for action in ("download",
-                           "parse", "relate", "makeresources",
-                           "generate", "toc", "news", "frontpage"):
-                if action in ("makeresources", "frontpage"):
-                    argscopy = list(args)
-                    argscopy.extend(_filter_argv_options(argv))
-                    argscopy.insert(0, action)
-                    argscopy.insert(0, "all")
-                    results[action] = run(argscopy)
-                else:
-                    results[action] = OrderedDict()
-                    for classname in classnames:
-                        alias = enabled_aliases[classname]
+            elif action == 'all':
+                classnames = _setup_classnames(enabled, classname)
+                results = OrderedDict()
+                for action in ("download",
+                               "parse", "relate", "makeresources",
+                               "generate", "toc", "news", "frontpage"):
+                    if action in ("makeresources", "frontpage"):
                         argscopy = list(args)
                         argscopy.extend(_filter_argv_options(argv))
-                        if (action in ("parse", "relate", "generate") and
-                                "--all" not in argscopy):
-                            argscopy.append("--all")
                         argscopy.insert(0, action)
-                        argscopy.insert(0, classname)
-                        results[action][alias] = run(argscopy)
-            return results
-        else:
-            if classname == "all":
-                ret = []
-                for alias, classname in enabled.items():
-                    argv_copy = list(argv)
-                    argv_copy[0] = alias
-                    ret.append(_run_class(enabled, argv_copy))
-                return ret
+                        argscopy.insert(0, "all")
+                        results[action] = run(argscopy)
+                    else:
+                        results[action] = OrderedDict()
+                        for classname in classnames:
+                            alias = enabled_aliases[classname]
+                            argscopy = list(args)
+                            argscopy.extend(_filter_argv_options(argv))
+                            if (action in ("parse", "relate", "generate") and
+                                    "--all" not in argscopy):
+                                argscopy.append("--all")
+                            argscopy.insert(0, action)
+                            argscopy.insert(0, classname)
+                            results[action][alias] = run(argscopy)
+                return results
             else:
-                return _run_class(enabled, argv)
-
+                if classname == "all":
+                    ret = []
+                    for alias, classname in enabled.items():
+                        argv_copy = list(argv)
+                        argv_copy[0] = alias
+                        try:
+                            ret.append(_run_class(enabled, argv_copy))
+                        except Exception as e:
+                            (alias, command, args) = _filter_argv(argv_copy)
+                            log.error("%s %s failed: %s" % (command, alias, e))
+                    return ret
+                else:
+                    return _run_class(enabled, argv)
+    finally:
+        shutdown_logger()
 
 def enable(classname):
     """Registers a class by creating a section for it in the
@@ -1093,11 +1109,7 @@ def _instantiate_class(cls, configfile="ferenda.ini", argv=[]):
                               configfile,
                               argv, cascade=True)
     classcfg = getattr(globalcfg, cls.alias)
-    inst.config = classcfg
-    inst.store = inst.documentstore_class(
-        classcfg.datadir + os.sep + inst.alias,
-        downloaded_suffix=inst.downloaded_suffix,
-        storage_policy=inst.storage_policy)
+    inst.config = classcfg # magically creates inst.store
     return inst
 
 
@@ -1204,6 +1216,8 @@ def _list_class_usage(cls):
     res = OrderedDict()
     for attrname in dir(cls):
         attr = getattr(cls, attrname)
+        if type(attr).__module__.startswith("rdflib."):
+            continue
         if hasattr(attr, "runnable"):
             doc = attr.__doc__
             if doc:
@@ -1233,6 +1247,8 @@ def _filter_argv(args):
         command = args[1]
     if len(args) > 2:
         for arg in args[2:]:
+            if isinstance(arg, bytes):
+                arg = arg.decode("utf-8") # FIXME: duplicated code of LayeredConfig._load_commandline
             if not arg.startswith("--"):
                 commandargs.append(arg)
     return (alias, command, commandargs)
@@ -1353,9 +1369,11 @@ def _filepath_to_urlpath(path, keep_segments=2):
     """
     # data/repo/rsrc/js/main.js, 3 -> repo/rsrc/js/main.js
     # /var/folders/tmp4q6b1g/rsrc/resources.xml, 1 -> rsrc/resources.xml
-    urlpath = os.sep.join(path.split(os.sep)[-(keep_segments + 1):])
+    # C:\docume~1\owner\locals~1\temp\tmpgbyuk7\rsrc\css\test.css, 2 - rsrc/css/test.css
+    path = path.replace(os.sep, "/")
+    urlpath = "/".join(path.split("/")[-(keep_segments + 1):])
     # print("_filepath_to_urlpath (%s): %s -> %s" % (keep_segments, path, urlpath))
-    return urlpath.replace(os.sep, "/")
+    return urlpath
 
 
 def _preflight_check(log, verbose=False):
@@ -1367,7 +1385,7 @@ def _preflight_check(log, verbose=False):
         ('bs4', '4.3.0', True),
         # ('lxml', '3.2.0', True), # has no top level __version__ property
         ('rdflib', '4.0', True),
-        ('html5lib', '1.0b1', True),
+        ('html5lib', '0.99', True),
         ('requests', '1.2.0', True),
         ('six', '1.4.0', True),
         ('jsmin', '2.0.2', True),
