@@ -1,10 +1,10 @@
 # flake8: noqa
 from rdflib import Namespace
 RPUBL = Namespace('http://rinfo.lagrummet.se/ns/2008/11/rinfo/publ#')
-from .swedishlegalsource import SwedishLegalSource, SwedishLegalStore
+from .swedishlegalsource import SwedishLegalSource, SwedishLegalStore, SwedishCitationParser
 from .regeringen import Regeringen
 from .riksdagen import Riksdagen
-from .trips import Trips
+from .trips import Trips, NoMoreLinks
 from .arn import ARN
 from .direktiv import Direktiv
 from .ds import Ds
