@@ -11,6 +11,8 @@ import logging
 import tempfile
 # 3rdparty libs
 
+from six import unichr as chr
+
 # needed early
 from ferenda import util
 
@@ -1206,7 +1208,9 @@ class LegalRef:
                        'MIG': '/publ/rattsfall/mig/',
                        'AD': '/publ/rattsfall/ad/',
                        'MD': '/publ/rattsfall/md/',
-                       'FÖD': '/publ/rattsfall/fod/'}
+                       'FÖD': '/publ/rattsfall/fod/',
+                       'HFD': '/publ/rattsfall/hfd/',
+        }
 
         # res = self.baseuri_attributes['baseuri']
         if 'nja' in attributes:
@@ -1220,7 +1224,11 @@ class LegalRef:
         if 'lopnr' in attributes and ":" in attributes['lopnr']:
             (attributes['ar'], attributes['lopnr']) = lopnr.split(":", 1)
 
-        if attributes['domstol'] == 'NJA':
+        rf = attributes['njarattsfall'] if 'njarattsfall' in attributes else attributes['rattsfall']
+        if "not" in rf:
+            ordinal = attributes['sidnr'] if 'sidnr' in attributes else attributes['lopnr']
+            res += "%s/not/%s" % (attributes['ar'], ordinal)
+        elif attributes['domstol'] == 'NJA':
             # FIXME: URIs should be based on publikationsordinal, not
             # pagenumber (which this in effect is) - but this requires
             # a big lookup table/database/graph with
