@@ -6,29 +6,30 @@ import sys
 import os
 import re
 import codecs
-from datetime import datetime
+from datetime import datetime, timedelta, date
 from six import text_type as str
-from six.moves.urllib_parse import urljoin
+from six.moves.urllib_parse import urljoin, urlencode
 
 import requests
 from bs4 import BeautifulSoup
 from rdflib import URIRef
-from rdflib import RDFS
+from rdflib import RDFS, XSD
 
 from ferenda import PDFDocumentRepository
 from ferenda import DocumentEntry
+from ferenda import FSMParser
 from ferenda import Describer
 from ferenda import util
-from ferenda.elements import Body
-from ferenda.elements import Paragraph
-from ferenda.elements import Page
+from ferenda.compat import OrderedDict
+from ferenda.elements import Body, Paragraph, Section, CompoundElement, SectionalElement
 from ferenda.pdfreader import PDFReader
-from ferenda.pdfreader import Page
+from ferenda.pdfreader import Page, Textbox
 from ferenda.decorators import recordlastdownload, downloadmax
 from . import SwedishLegalSource
+from .swedishlegalsource import offtryck_parser, offtryck_gluefunc, PreambleSection, UnorderedSection
 
 
-class Regeringen(SwedishLegalSource, PDFDocumentRepository):
+class Regeringen(SwedishLegalSource):
     DS = 1
     KOMMITTEDIREKTIV = 2
     LAGRADSREMISS = 3
@@ -44,10 +45,24 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
 
     document_type = None  # subclasses must override
     start_url = None
-    start_url_template = "http://www.regeringen.se/sb/d/107/a/136/action/showAll/sort/byDate/targetDepartment/archiveDepartment?d=107&action=search&query=&type=advanced&a=136&sort=byDate&docTypes=%(doctype)s"
+    start_url_template = 'http://www.regeringen.se/sb/d/107/a/136/action/showAll/sort/byDate/targetDepartment/archiveDepartment'
+    start_url_template_parameters = [('d', '107'),
+                                     ('a', '136'),
+                                     ('action', 'search'),
+                                     ('bottomQuery', ''),
+                                     ('dateRange', '1'),
+                                     ('query', ''),
+                                     ('queryLogic', '1'),
+                                     ('sortOrder', '1'),
+                                     ('sort', 'byDate'),
+                                     ('type', 'advanced')]
     downloaded_suffix = ".html"  # override PDFDocumentRepository
     source_encoding = "latin-1"
+    storage_policy = "dir"
 
+
+    session = None
+    
     @recordlastdownload
     def download(self, basefile=None):
         if basefile:
@@ -55,19 +70,42 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
 
         # if self.config.lastdownloaded:
         #     FIXME: use this to create a time-filtered start_url
-        start_url = self.start_url_template % {'doctype': self.document_type}
+        today = datetime.today()
+        # we use ordereddict and lists of 2-tuples to get a
+        # predictable ordering of parameters when constructing the URL
+        p = OrderedDict(self.start_url_template_parameters)
+        p.update([('dateRangeFromDay', today.day), # 18b
+                  ('dateRangeFromMonth', today.month), # 3
+                  ('dateRangeFromYear', today.year - 1999), # 15
+                  ('dateRangeToDay', today.day),
+                  ('dateRangeToMonth', today.month),
+                  ('dateRangeToYear', today.year - 1998), # 16
+                  ('docTypes', self.document_type)])
+        if self.config.lastdownload and not self.config.refresh:
+            last = self.config.lastdownload - timedelta(days=1)
+            self.log.debug("Only downloading documents published on or after %s"
+                           % last)
+            p.update([('dateRange', '4'),
+                      ('dateRangeFromDay', last.day),
+                      ('dateRangeFromMonth', last.month),
+                      ('dateRangeFromYear', last.year - 1998)])
+                      
+                      
+        start_url = self.start_url_template + "?" + urlencode(list(p.items()))
         self.log.info("Starting at %s" % start_url)
+        self.session = requests.session()
         for basefile, url in self.download_get_basefiles(start_url):
             self.download_single(basefile, url)
 
     @downloadmax
     def download_get_basefiles(self, url):
         done = False
-        pagecnt = 1
+        pagecount = 1
         # existing_cnt = 0
         while not done:
-            self.log.info('Result page #%s (%s)' % (pagecnt, url))
-            resp = requests.get(url)
+            self.log.info('Result page #%s (%s)' % (pagecount, url))
+            # resp = requests.get(url)
+            resp = self.session.get(url)
             mainsoup = BeautifulSoup(resp.text)
             for link in mainsoup.find_all(href=re.compile("/sb/d/108/a/")):
                 desc = link.find_next_sibling("span", "info").get_text(strip=True)
@@ -109,8 +147,9 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
 
                 yield basefile, urljoin(url, link['href'])
 
-            pagecnt += 1
-            next = mainsoup.find("a", text="Nästa sida")
+            pagecount += 1
+            # next = mainsoup.find("a", text="Nästa sida")
+            next = mainsoup.find("a", text=str(pagecount))
             if next:
                 url = urljoin(url, next['href'])
             else:
@@ -118,7 +157,7 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
 
     def remote_url(self, basefile):
         # do a search to find the proper url for the document
-        templ = "http://www.regeringen.se/sb/d/107/a/136?query=s(basefile)&docTypes=s(doctype)&type=advanced&action=search"
+        templ = "http://www.regeringen.se/sb/d/107/a/136?query=%(basefile)s&docTypes=%(doctype)s&type=advanced&action=search"
         url = templ % {'doctype': self.document_type,
                        'basefile': basefile}
         soup = BeautifulSoup(requests.get(url).text)
@@ -156,14 +195,12 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
             docid = url.split("/")[-1]
             if existed:
                 if updated:
-                    self.log.debug(
-                        "%s existed, but a new ver was downloaded" % filename)
+                    self.log.info("%s: updated from %s" % (basefile, url))
                 else:
-                    self.log.debug(
-                        "%s is unchanged -- checking PDF files" % filename)
+                    self.log.debug("%s: %s is unchanged, checking PDF files" %
+                                   (basefile, filename))
             else:
-                self.log.debug(
-                    "%s did not exist, so it was downloaded" % filename)
+                        self.log.info("%s: downloaded from %s" % (basefile, url))
 
             soup = BeautifulSoup(codecs.open(filename, encoding=self.source_encoding))
             cnt = 0
@@ -208,9 +245,8 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
         entry.save()
 
         return updated or pdfupdated
-
+        
     def parse_metadata_from_soup(self, soup, doc):
-        doc.lang = "sv"
         d = Describer(doc.meta, doc.uri)
         d.rdftype(self.rdf_type)
         d.value(self.ns['prov'].wasGeneratedBy, self.qualified_class_name())
@@ -232,8 +268,13 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
                 value = dt.find_next_sibling("dd").get_text(strip=True)
                 if key == "Utgiven:":
                     try:
-                        d.value(self.ns['dct'].published,
-                                self.parse_swedish_date(value))
+                        dateval = self.parse_swedish_date(value)
+                        if type(dateval) == date:
+                            d.value(self.ns['dct'].issued, dateval)
+                        else:
+                            datatype = {util.gYearMonth: XSD.gYearMonth,
+                                        util.gYear: XSD.gYear}[type(dateval)]
+                            d.value(self.ns['dct'].issued, str(dateval), datatype=datatype)
                     except ValueError as e:
                         self.log.warning(
                             "Could not parse %s as swedish date" % value)
@@ -284,7 +325,7 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
 
                 if not rel_basefile:
                     self.log.warning(
-                        "Couldn't find rel_basefile (elementid #%s) among %r" % (elementid, infospans))
+                        "%s: Couldn't find rel_basefile (elementid #%s) among %r" % (doc.basefile, elementid, infospans))
                     continue
                 if elementid == "legStep1":
                     subjUri = self.canonical_uri(
@@ -320,17 +361,101 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
         # find pdf file names in order
 
     def parse_document_from_soup(self, soup, doc):
+        def _check_differing(describer, predicate, newval):
+            if describer.getvalue(predicate) != newval:
+                self.log.warning("%s: HTML page: %s is %r, document: it's %r" %
+                                 (doc.basefile,
+                                  d.graph.qname(predicate),
+                                  describer.getvalue(predicate),
+                                  newval))
+                # remove old val
+                d.graph.remove((d._current(),
+                                predicate,
+                                d.graph.value(d._current(), predicate)))
+                d.value(predicate, newval)
+
+        # reset global state
+        PreambleSection.counter = 0
+        UnorderedSection.counter = 0
+
         pdffiles = self.find_pdf_links(soup, doc.basefile)
         if not pdffiles:
             self.log.error(
                 "%s: No PDF documents found, can't parse anything" % doc.basefile)
             return None
-
+            
         doc.body = self.parse_pdfs(doc.basefile, pdffiles)
+
+        # do some post processing. First loop through leading
+        # textboxes and try to find dct:identifier, dct:title and
+        # dct:issued (these should already be present in doc.meta,
+        # but the values in the actual document should take
+        # precendence
+        d = Describer(doc.meta, doc.uri)
+        title_found = False
+        for idx, element in enumerate(doc.body):
+            if not isinstance(element, Textbox):
+                continue
+            str_element = str(element).strip()
+            # print("examining %s..." % str_element[:40])
+            # dct:identifier
+            m = self.re_basefile_lax.search(str_element)
+            if m:
+                _check_differing(d, self.ns['dct'].identifier, "Prop. " + m.group(1))
+            # dct:title
+            if element.getfont()['size'] == '20' and not title_found:
+                # sometimes part of the the dct:identifer (eg " Prop."
+                # or " 2013/14:51") gets mixed up in the title
+                # textbox. Remove those parts if we can find them.
+                if " Prop." in str_element:
+                    str_element = str_element.replace(" Prop.", "").strip()
+                if self.re_basefile_lax.search(str_element):
+                    str_element = self.re_basefile_lax.sub("", str_element)
+                _check_differing(d, self.ns['dct'].title, str_element)
+                title_found = True
+            # dct:issued
+            if str_element.startswith("Stockholm den"):
+                pubdate = self.parse_swedish_date(str_element[13:])
+                _check_differing(d, self.ns['dct'].issued, pubdate)
+
+        # then maybe look for the section named Författningskommentar
+        # (or similar), identify each section and which proposed new
+        # regulation it refers to)
+        for i, element in enumerate(doc.body):
+            if isinstance(element, Section) and (element.title == "Författningskommentar"):
+                for j, subsection in enumerate(element):
+                    if hasattr(subsection, 'title'):
+                        law = subsection.title # well, find out the id (URI) from the title -- possibly using legalref
+                        for k, p in enumerate(subsection):
+                            # find out individual paragraphs, create uris for
+                            # them, and annotate the first textbox that might
+                            # contain commentary (ideally, identify set of
+                            # textboxes that comment on a particular
+                            # identifiable section and wrap them in a
+                            # CommentaryOn container)
+                            pass
+                            # print("%s,%s,%s: %s" % (i,j,k,repr(p)))
+                
+                
+        # then maybe look for inline references ("Övervägandena finns
+        # i avsnitt 5.1 och 6" using CitationParser)
+                
         return doc
 
     def sanitize_identifier(self, identifier):
-        return identifier
+        pattern = {self.KOMMITTEDIREKTIV: "%s. %s:%s",
+                   self.DS: "%s %s:%s",
+                   self.PROPOSITION: "%s. %s/%s:%s",
+                   self.SKRIVELSE: "%s. %s/%s:%s",
+                   self.SOU: "%s %s:%s",
+                   self.SO: "%s %s:%s"}
+        
+        try: 
+            parts = re.split("[\.:/ ]+", identifier.strip())
+            return pattern[self.document_type] % tuple(parts)
+        except:
+            self.log.warning("Couldn't sanitize identifier %s" % identifier)
+            return identifier
 
     def find_pdf_links(self, soup, basefile):
         pdffiles = []
@@ -342,36 +467,97 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
                 if not m:
                     continue
                 pdfbasefile = m.group(1)
-                pdffiles.append(pdfbasefile)
-        return pdffiles
+                pdffiles.append((pdfbasefile, link.string))
+        selected = self.select_pdfs(pdffiles)
+        self.log.debug("selected %s out of %d pdf files" % (", ".join(selected), len(pdffiles)))
+        return selected
 
-    def parse_pdfs(self, basefile, pdffiles):
-        doc = Body()
-        for pdffile in pdffiles:
-            # FIXME: downloaded_path must be more fully mocked
-            # (support attachments) by testutil.RepoTester. In the
-            # meantime, we do some path munging ourselves
 
-            pdf_path = self.store.downloaded_path(basefile).replace("index.html", pdffile)
-            intermediate_path = self.store.intermediate_path(basefile, attachment=pdffile)
-            intermediate_dir = os.path.dirname(intermediate_path)
-            try:
-                pdf = self.parse_pdf(pdf_path, intermediate_dir)
-                for page in pdf:
-                    pass
-                    # page.crop(left=50,top=0,bottom=900,right=700)
-                doc.append(pdf)
-            except ValueError:
-                (exc_type, exc_value, exc_trackback) = sys.exc_info()
-                self.log.warning("Ignoring exception %s (%s), skipping PDF %s" %
-                                 (exc_type, exc_value, pdffile))
-        return doc
+    def select_pdfs(self, pdffiles):
+        """Given a list of (pdffile, linktext) tuples, return only those pdf
+        files we need to parse (by filtering out duplicates etc).
+        """
+        cleanfiles = []
 
+        # 1. Simplest case: One file obviously contains all of the text
+        for pdffile, linktext in pdffiles:
+            if "hela dokumentet" in linktext or "hela betänkandet" in linktext:
+                return [pdffile] # we're immediately done
+
+        # 2. Filter out obviously extraneous files
+        for pdffile, linktext in pdffiles:
+            if "hela dokumentet" in linktext or "hela betänkandet" in linktext:
+                return [pdffile] # we're immediately done
+            if (linktext.startswith("Sammanfattning ") or
+                linktext.startswith("Remissammanställning") or
+                linktext.startswith("Sammanställning över remiss") or
+                "remissinstanser" in linktext):
+                pass # don't add to cleanfiles
+            else:
+                cleanfiles.append((pdffile,linktext))
+
+        # 3. Attempt to see if we have one complete file + several
+        # files with split-up content
+        linktexts = [x[1] for x in cleanfiles]
+        commonprefix = os.path.commonprefix(linktexts)
+        if commonprefix:
+            for pdffile, linktext in cleanfiles:
+                # strip away the last filetype + size paranthesis
+                linktext = re.sub(" \(pdf [\d\,]+ [kM]B\)", "", linktext)
+                # and if we remove the commonprefix, do we end up with nothing?
+                if linktext.replace(commonprefix, "") == "":
+                    # then this is probably a complete file
+                    return [pdffile]
+
+        # 4. Base case: We return it all
+        return [x[0] for x in cleanfiles]
+    
+        
     def parse_pdf(self, pdffile, intermediatedir):
         pdf = PDFReader()
-        pdf.read(pdffile, intermediatedir)
+        # By default, don't create and manage PDF backgrounds files
+        # (takes forever, we don't use them yet)
+        if self.config.compress == "bz2":
+            keep_xml = "bz2"
+        else:
+            keep_xml = True
+        pdf.read(pdffile, intermediatedir, images=self.config.pdfimages, keep_xml=keep_xml)
         return pdf
 
+                
+    def parse_pdfs(self, basefile, pdffiles):
+        body = None
+        gluefunc = offtryck_gluefunc
+        for pdffile in pdffiles:
+            pdf_path = self.store.downloaded_path(basefile, attachment=pdffile)
+            intermediate_path = self.store.intermediate_path(basefile, attachment=pdffile)
+            intermediate_dir = os.path.dirname(intermediate_path)
+            # case 1: intermediate path does not exist and that's ok
+            # case 2: intermediate path exists alongside downloaded_path
+            pdf = self.parse_pdf(pdf_path, intermediate_dir)
+            debug = False
+            if debug:
+                outputfile = pdf_path+".marked.pdf"
+                pdf.drawboxes(gluefunc, outputfile)
+
+            if self.document_type == self.PROPOSITION:
+                preset = 'proposition'
+            elif self.document_type == self.SOU:
+                preset = 'sou'
+            elif self.document_type == self.DS:
+                preset = 'ds'
+            elif self.document_type == self.KOMMITTEDIREKTIV:
+                preset = 'dir'
+            else:
+                preset = 'default'
+            parser = offtryck_parser(preset=preset)
+            parser.debug = os.environ.get('FERENDA_FSMDEBUG', False)
+            body = parser.parse(pdf.textboxes(gluefunc, pageobjects=True))
+            pdf[:] = body[:]
+            pdf.tagname = "body"
+        return pdf
+
+
     def create_external_resources(self, doc):
         """Optionally create external files that go together with the
         parsed file (stylesheets, images, etc). """
@@ -385,33 +571,38 @@ class Regeringen(SwedishLegalSource, PDFDocumentRepository):
         # 1.2 create static CSS
         fp = open(cssfile, "w")
         # 1.3 create css for fontspecs and pages
-        for pdf in doc.body:
-            assert isinstance(pdf, PDFReader)
-            for spec in list(pdf.fontspec.values()):
-                fp.write(".fontspec%s {font: %spx %s; color: %s;}\n" %
-                         (spec['id'], spec['size'], spec['family'], spec['color']))
+        # for pdf in doc.body:
+        pdf = doc.body
+        assert isinstance(pdf, PDFReader) # this is needed to get fontspecs and other things
+        for spec in list(pdf.fontspec.values()):
+            fp.write(".fontspec%s {font: %spx %s; color: %s;}\n" %
+                     (spec['id'], spec['size'], spec['family'], spec['color']))
 
         # 2 Copy all created png files to their correct locations
         totcnt = 0
         src_base = os.path.dirname(self.store.intermediate_path(doc.basefile))
-        for pdf in doc.body:
-            pdf_src_base = src_base + "/" + os.path.splitext(os.path.basename(pdf.filename))[0]
 
-            cnt = 0
-            for page in pdf:
-                totcnt += 1
-                cnt += 1
-                src = "%s%03d.png" % (pdf_src_base, page.number)
-                # 4 digits, compound docs can be over 1K pages
-                attachment = "%04d.png" % (totcnt)
-                dest = self.store.parsed_path(doc.basefile,
-                                              attachment=attachment)
+        pdf_src_base = src_base + "/" + os.path.splitext(os.path.basename(pdf.filename))[0]
 
+        cnt = 0
+        for page in pdf:
+            totcnt += 1
+            cnt += 1
+            # src = "%s%03d.png" % (pdf_src_base, page.number)
+            src = "%s%03d.png" % (pdf_src_base, cnt)
+
+            # 4 digits, compound docs can be over 1K pages
+            attachment = "%04d.png" % (totcnt)
+            dest = self.store.parsed_path(doc.basefile,
+                                          attachment=attachment)
+
+            # If running under RepoTester, the source PNG files may not exist.
+            if os.path.exists(src):
                 if util.copy_if_different(src, dest):
                     self.log.debug("Copied %s to %s" % (src, dest))
 
-                fp.write("#page%03d { background: url('%s');}\n" %
-                         (cnt, os.path.basename(dest)))
+            fp.write("#page%03d { background: url('%s');}\n" %
+                     (cnt, os.path.basename(dest)))
 
     # Not used right now
     def parse_pdf_complex(self, pdffile, intermediatedir):
